'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformString = transformString;
exports.transformObject = transformObject;
exports.setTransforms = setTransforms;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @typedef {Object} ObjectTransformV2
 * @property {Object} object - format of output object where values are string templates
 */

/**
 * @typedef {Object} StringTransformV2
 * @property {String} from - matching literal or regex pattern
 * @property {Boolean} global - whether or not to use global regex replace
 * @property {String} to - replacement literal or regex replace pattern
 */

/**
 * Convert UI Schema v1 string transform into a UI Schema v2 string transform
 * @param {Object} uis1Transform - UI Schema v1 string trasnform
 * @returns {StringTransformV2} UI Schema v2 string transform
 * @example
 *
 *   const uis1Transform = {'\n': ','}
 *   transformString(uis1Transform)
 *   > {
 *   >   from: '\n',
 *   >   to: ',',
 *   >   global: true
 *   > }
 */
function transformString(uis1Transform) {
  var key = _lodash2.default.keys(uis1Transform)[0];
  return {
    from: key,
    to: uis1Transform[key],
    global: true
  };
}

/**
 * Convert UI Schema v1 object transform into a UI Schema v2 object transform
 * @param {Object} uis1Transform - UI Schema v1 object transform
 * @returns {ObjectTransformV2} UI Schema v2 object transform
 * @example
 *
 * const uis1Transform = {
 *   id: 'value',
 *   something: '\'someliteral\'',
 *   label: 'label',
 *   otherId: 'id'
 * }
 * transformObject(uis1Transform)
 * > {
 * >   object: {
 * >     id: '${value}',
 * >     something: 'someliteral',
 * >     label: '${label}',
 * >     otherId: '${id}'
 * >   }
 * > }
 */
function transformObject(uis1Transform) {
  var keywords = ['value', 'id', 'label'];
  return {
    object: _lodash2.default.reduce(uis1Transform, function (result, value, key) {
      if (value && value.split && value.split('\'').length === 3) {
        value = value.split('\'')[1];
      }
      if (_lodash2.default.indexOf(keywords, value) !== -1) {
        value = '${' + value + '}';
      }
      result[key] = value;
      return result;
    }, {})
  };
}

function setTransforms(newField, oldField, logger) {
  var transforms = {};[{ fn: transformObject, key: 'write', val: _lodash2.default.get(oldField, 'modelTransform.toObject') }, { fn: transformString, key: 'write', val: _lodash2.default.get(oldField, 'modelTransform.replace') }, { fn: transformObject, key: 'read', val: _lodash2.default.get(oldField, 'viewTransform.toObject') }, { fn: transformString, key: 'read', val: _lodash2.default.get(oldField, 'viewTransform.replace') }].forEach(function (data) {
    if (data.val) {
      transforms[data.key] = [data.fn(data.val)];
    }
  });

  if (!_lodash2.default.isEmpty(transforms)) _lodash2.default.set(newField, 'transforms', transforms);

  return newField;
}